#!/usr/bin/env python3
"""
Static TF publisher example
In this code, we are going to publish a static transformation into ROS.
It represents a "protrusion" dangling from our (fake) robot. The protrusion
is fixed, that is why static tf is used instead of "normal" (dynamic) tf.
"""
from __future__ import division  # remember that in Python 2.7 normally 3 / 2 = 1
import rospy
import sys
import tf2_ros  # package for TF in ROS
import tf  # package containing functions to work with geometrical transformations
from geometry_msgs.msg import TransformStamped  # this message is the basis of TFs
import numpy as np

if __name__ == "__main__":
    rospy.init_node("bot_body_tf")
    if len(sys.argv) < 2:
        # The offset of the protrusion has to be specified via command line arguments
        rospy.logerr("No position arguments provided! Please, write some two numbers (representing x/y offset in cm) after the script name when running from console.")
        sys.exit(-1)
    """
    Get the name of robot frame from ROS parameter server.
    This is considered a good habit in ROS. Instead of hardcoding the names
    into the script itself, you should store the names into the parameter server.
    Otherwise, if at any point you would like to change the frame name,
    you would have to search for all the places where it is used and change it.
    Not the most fun thing you can do.
    """
    botFrame = rospy.get_param('bot_frame', "bot")
    # get the name of protrusion frame from ROS parameter server
    protrusionFrame = rospy.get_param('protrusion_frame', "protrusion")
    # Create and instance of "StaticTransformBroadcaster"
    # no points for guessing what it does (an example of good naming style)
    broadcaster = tf2_ros.StaticTransformBroadcaster()

    # create an empty TF message
    static_tf = TransformStamped()

    # first, fill out the message header
    static_tf.header.stamp = rospy.Time.now()  # message generation time
    static_tf.header.frame_id = botFrame  # parent frame id (string/name)
    static_tf.child_frame_id = protrusionFrame  # child frame id

    """
    In the following, we specify the actual transformation,
    consisting of translation and rotation.
    Remember, we work in 3D, even though this example uses only the 2D part
    (assumes we are working on a plane).
    Thus, the translation part is represented by a 3D vector.
    The rotation is a bit more complex (I guess only those who know quaternions
    will get the joke) - it is represented by what is called "a quaternion".
    Quaternion is simply one of the ways a rotation in 3D space
    can be represented. If you don't know about quaternions,
    don't worry, the "tf" library contains functions to convert to and from
    quaterions from other (more human readable) forms (see example below).
    """
    static_tf.transform.translation.x = float(sys.argv[1]) / 100
    static_tf.transform.translation.y = float(sys.argv[2]) / 100
    static_tf.transform.translation.z = 0  # we don't use the Z-axis in this demo

    # rotation is also not used (much) but it is present here for illustration
    quat = tf.transformations.quaternion_from_euler(0, 0, np.deg2rad(45))
    static_tf.transform.rotation.x = quat[0]
    static_tf.transform.rotation.y = quat[1]
    static_tf.transform.rotation.z = quat[2]
    static_tf.transform.rotation.w = quat[3]

    # finally, we broad cast the static transformation message
    broadcaster.sendTransform(static_tf)
    rospy.spin()
